#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Box.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Image.H>
#include <FL/Fl_PNG_Image.H>

#include <pthread.h>
#include "joystick.h"

Fl_Double_Window*   window = NULL;
Fl_Box*             boxState = NULL;

Fl_Group*           grpJoystick = NULL;
Fl_Box*             boxJoystick = NULL;
Fl_Box*             boxAxis1 = NULL;
Fl_Box*             boxAxis2 = NULL;
Fl_Box*             boxAxisBtns[4] = {NULL};
Fl_Box*             boxUpperBtns[4] = {NULL};
Fl_Box*             boxRightBtns[4] = {NULL};
Fl_Box*             boxCenterBtns[2] = {NULL};

pthread_t           pth_id = NULL;
bool                thread_lived = true;

void window_cb( Fl_Widget* w, void* p )
{
#ifdef DEBUG
    printf("Thread termination signal ...\n");
#endif
    thread_lived = false;

    pthread_join( pth_id, NULL );

#ifdef DEBUG
    printf("Thread done.\n");
#endif

    window->hide();
}

void* pthread_loop( void* arg )
{
    while( window->shown() == 0 )
    {
        Fl::wait();
    }

    HWND hCur = fl_xid( window );

    if ( InitDirectInput( hCur ) != S_OK )
    {
        boxState->label("DirectX8 input initialize failure !");
        return NULL;
    }
    else
    {
        boxState->label("DirectX8 input initialized.");
    }

    JOYSTICK_OPTIONS joyOps;
    JOYSTATE         joyStat;

    GetJoystickOptions( &joyOps );

    while( thread_lived == true )
    {
        if ( UpdateInputState() == S_OK )
        {
            GetJoystickState( &joyStat );

            boxJoystick->redraw();

            for(int cnt=0; cnt<4; cnt++)
            {
                boxAxisBtns[cnt]->hide();
            }

            if ( joyStat.lX < -1 )
            {
                boxAxisBtns[0]->show();
            }
            else
            if ( joyStat.lX > 1 )
            {
                boxAxisBtns[1]->show();
            }

            if ( joyStat.lY < -1 )
            {
                boxAxisBtns[2]->show();
            }
            else
            if ( joyStat.lY > 1 )
            {
                boxAxisBtns[3]->show();
            }

            for(int cnt=0; cnt<4; cnt++)
            {
                boxAxisBtns[cnt]->redraw();
            }


            // Check buttons ...
            for(int cnt=0; cnt<4; cnt++)
            {
                boxRightBtns[cnt]->hide();
                boxUpperBtns[cnt]->hide();
            }

            for(int cnt=0; cnt<2; cnt++ )
            {
                boxCenterBtns[cnt]->hide();
            }

            if ( joyStat.rgbButtons[0] > 0 )
            {
                boxRightBtns[0]->show();
            }

            if ( joyStat.rgbButtons[1] > 0 )
            {
                boxRightBtns[2]->show();
            }

            if ( joyStat.rgbButtons[2] > 0 )
            {
                boxRightBtns[3]->show();
            }

            if ( joyStat.rgbButtons[3] > 0 )
            {
                boxRightBtns[1]->show();
            }

            if ( joyStat.rgbButtons[4] > 0 )
            {
                boxUpperBtns[0]->show();
            }

            if ( joyStat.rgbButtons[5] > 0 )
            {
                boxUpperBtns[1]->show();
            }

            if ( joyStat.rgbButtons[6] > 0 )
            {
                boxUpperBtns[2]->show();
            }

            if ( joyStat.rgbButtons[7] > 0 )
            {
                boxUpperBtns[3]->show();
            }

            if ( joyStat.rgbButtons[8] > 0 )
            {
                boxCenterBtns[0]->show();
            }

            if ( joyStat.rgbButtons[9] > 0 )
            {
                boxCenterBtns[1]->show();
            }

            for(int cnt=0; cnt<4; cnt++)
            {
                boxRightBtns[cnt]->redraw();
                boxUpperBtns[cnt]->redraw();
            }

            for(int cnt=0; cnt<2; cnt++ )
            {
                boxCenterBtns[cnt]->redraw();
            }

            Fl::flush();
        }

        Sleep(30);
    }

    FreeDirectInput();

    return NULL;
}

void applyNormalTheme2FlWidget( Fl_Widget* w )
{
    if ( w != NULL )
    {
        w->box( FL_FLAT_BOX );
        w->color( FL_DARK3 );
        w->labelcolor( FL_WHITE );
    }
}

void applyConsoleTheme2FlWidget( Fl_Widget* w )
{
    if ( w != NULL )
    {
        w->box( FL_DOWN_BOX );
        w->align( FL_ALIGN_INSIDE | FL_ALIGN_TOP_LEFT );
        w->color( FL_BLACK );
        w->labelcolor( FL_GREEN );
    }
}

void applyEmptyTheme2FlWidget( Fl_Widget* w )
{
    if ( w != NULL )
    {
        w->box( FL_FLAT_BOX );
        w->align( FL_ALIGN_INSIDE );
        w->color( FL_DARK3 );
        w->labelcolor( FL_WHITE );
    }
}

void applyRectBtnTheme2FlWidget( Fl_Widget* w )
{
    if ( w != NULL )
    {
        //w->box( FL_BORDER_FRAME );
        w->box( _FL_GTK_ROUND_UP_BOX );
        w->align( FL_ALIGN_INSIDE );
        w->color( 0xFF000022 );
        w->labelcolor( FL_WHITE );
    }
}

int main(int argc, char ** argv)
{
    Fl::scheme( "GTK+" );

    window = new Fl_Double_Window (640, 440, "FLTK DirectX8 input joystick test");
    if ( window != NULL )
    {
        window->callback( window_cb, NULL );
        applyNormalTheme2FlWidget( (Fl_Widget*)window );
        window->begin();

            boxState = new Fl_Box( 10,10, 620, 20, "--Ready--");
            if ( boxState != NULL )
            {
                applyConsoleTheme2FlWidget( (Fl_Widget*)boxState );
            }

            grpJoystick = new Fl_Group(0,40,640,397);
            if ( grpJoystick != NULL )
            {
                grpJoystick->begin();
            }

                boxJoystick = new Fl_Box( 0, 40, 640, 397 );
                if ( boxJoystick != NULL )
                {
                    applyEmptyTheme2FlWidget( (Fl_Widget*)boxJoystick );

                    Fl_PNG_Image* imgLoader = new Fl_PNG_Image( "joystick-38228_640.png" );
                    if ( imgLoader != NULL )
                    {
                        boxJoystick->image( imgLoader );
                    }
                    else
                    {
                        boxJoystick->label( "Error: joystick-38228_640.png file not found !" );
                    }
                }

                int base_x = 0;
                int base_y = 40;


                // Left , Right, Up, Dn,
                boxAxisBtns[0] = new Fl_Box( base_x + 74, base_y + 128, 47, 37, "@<-" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxAxisBtns[0] );
                boxAxisBtns[0]->hide();

                boxAxisBtns[1] = new Fl_Box( base_x + 140, base_y + 128, 47, 37, "@->" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxAxisBtns[1] );
                boxAxisBtns[1]->hide();

                boxAxisBtns[2] = new Fl_Box( base_x + 112, base_y + 93, 37, 47, "@2<-" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxAxisBtns[2] );
                boxAxisBtns[2]->hide();

                boxAxisBtns[3] = new Fl_Box( base_x + 112, base_y + 154, 37, 47, "@2->" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxAxisBtns[3] );
                boxAxisBtns[3]->hide();


                // right 4 buttons
                boxRightBtns[0] = new Fl_Box( base_x + 438, base_y + 123, 48, 48, "@square" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxRightBtns[0] );
                boxRightBtns[0]->hide();

                boxRightBtns[1] = new Fl_Box( base_x + 531, base_y + 123, 48, 48, "@circle" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxRightBtns[1] );
                boxRightBtns[1]->hide();

                boxRightBtns[2] = new Fl_Box( base_x + 486, base_y + 77, 48, 48, "@2<");
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxRightBtns[2] );
                boxRightBtns[2]->hide();

                boxRightBtns[3] = new Fl_Box( base_x + 486, base_y + 170, 48, 48, "@1+" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxRightBtns[3] );
                boxRightBtns[3]->hide();

                // Upper 2 pair buttons
                boxUpperBtns[0] = new Fl_Box( base_x + 75, base_y + 0, 110, 25, "L1" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxUpperBtns[0] );
                boxUpperBtns[0]->hide();

                boxUpperBtns[1] = new Fl_Box( base_x + 75, base_y + 30, 110, 25, "L2" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxUpperBtns[1] );
                boxUpperBtns[1]->hide();

                boxUpperBtns[2] = new Fl_Box( base_x + 450, base_y + 0, 110, 25, "R1" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxUpperBtns[2] );
                boxUpperBtns[2]->hide();

                boxUpperBtns[3] = new Fl_Box( base_x + 450, base_y + 30, 110, 25, "R2" );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxUpperBtns[3] );
                boxUpperBtns[3]->hide();


                // Center two buttons
                boxCenterBtns[0] = new Fl_Box( base_x + 240, base_y + 147, 45, 20 );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxCenterBtns[0] );
                boxCenterBtns[0]->hide();

                boxCenterBtns[1] = new Fl_Box( base_x + 353, base_y + 147, 45, 20 );
                applyRectBtnTheme2FlWidget( (Fl_Widget*)boxCenterBtns[1] );
                boxCenterBtns[1]->hide();



            if ( grpJoystick != NULL )
            {
                grpJoystick->end();
            }

        window->end();
        window->show();

        // initialize a thread for sensing joystick poll !
        if ( pthread_create( &pth_id, NULL, pthread_loop, NULL ) != 0 )
        {
            return -1;
        }
    }

    return(Fl::run());
}
